#pragma once

#include "language.hpp"

namespace {

	class deplist : public set<fs::path> {
	public:
		void add_file(const fs::path &p) {
			std::error_code ec;
			auto cp = fs::canonical(p, ec);
			if (!ec)
				insert(cp);
		}

		fs::file_time_type deps_last_mod() const {
			try {
				auto mtims = *this | ranges::views::transform(
						[](const auto &p) { return fs::last_write_time(p); }
						);
				return std::reduce(
						mtims.begin(), mtims.end(),
						fs::file_time_type::min(),
						[](const auto &a, const auto &b) { return std::max(a, b); }
						);
			} catch (system_error &) {
				return fs::file_time_type::max();
			}
		}

		bool is_outdated(const fs::path &p) const {
			fs::file_time_type pmod;
			try {
				pmod = fs::last_write_time(p);
			} catch (system_error &) {
				return true;
			}

			if (auto m = deps_last_mod(); m > fs::last_write_time(p))
				return true;
			return false;
		}

		void load(const fs::path &p) {
			string buf;
			{
				auto f = unixfd::open(p.c_str(), O_RDONLY | O_NOCTTY);
				f.readall(buf);
			}
			for (auto l: buf | ranges::views::split('\n') | ranges::views::transform([](const auto &e) { return string_view(e.begin(), e.end()); }))
				if (!l.empty())
					insert(l);
		}

		void store(const fs::path &p) {
			string buf;
			for (const auto &p: *this) {
				buf.append(p);
				buf.push_back('\n');
			}

			auto f = unixfd::open(p.c_str(), O_WRONLY | O_CREAT | O_NOCTTY, 0666);
			f.writeall(buf.data(), buf.size());
		}
	};

}
