#pragma once

#include "utils.hpp"

namespace {

class pkgconfig {
	set<string, std::less<>> _strings;
	set<fs::path, std::less<>> _paths;

	static const auto &_store(auto &s, auto &&v) {
		auto [i, e] = s.equal_range(v);
		if (i != e)
			return *i;
		i = s.emplace_hint(i, std::forward<decltype(v)>(v));
		return *i;
	}
	string_view _str(auto &&v) { return _store(_strings, std::forward<decltype(v)>(v)); }
	const fs::path &_path(auto &&v) { return _store(_paths, std::forward<decltype(v)>(v)); }

	map<string_view, bool> pkg_exist;
public:
	bool pkg_found(string_view pkg) {
		auto [i, e] = pkg_exist.equal_range(pkg);
		if (i != e)
			return i->second;

		child_process p({
			"pkg-config",
				"--exists",
				pkg,
		});
		p.detach_stdio();

		return pkg_exist.emplace_hint(i, _str(pkg), p.wait() == 0)->second;
	}

private:
	map<set<string_view>, vector<string_view>> pkg_opts;

public:
	vector<string_view> get_options(const set<string_view> &pkgs) {
		auto [i, e] = pkg_opts.equal_range(pkgs);
		if (i != e)
			return i->second;

		vector<string_view> cmd = {
			"pkg-config",
			"--cflags",
			"--libs",
		};
		for (auto p: pkgs)
			cmd.push_back(p);

		if (verbose()) {
			auto s = "invoking "s;
			for (const auto &a: cmd) {
				s.append(" '");
				s.append(a);
				s.append("'");
			}
			verbose(s);
		}

		child_process proc(cmd);
		proc.detach_stdin();

		auto stdout_pipe = proc.pipe(STDOUT_FILENO, 1);

		proc.spawn();

		string buf;
		stdout_pipe.readall(buf);

		if (proc.wait() != 0)
			throw runtime_error("pkg-config failed");

		auto args = *(_str(std::move(buf)) | ranges::views::split('\n')).begin() |
			ranges::views::split(' ') |
			ranges::views::transform([](const auto &e) { return string_view(e.begin(), e.end()); }) |
			ranges::views::filter([](const auto &e) { return !e.empty(); });

		return pkg_opts.emplace_hint(i, pkgs, vector<string_view>{args.begin(), args.end()})->second;
	}
};

}
